/*     DTN's Super SmartCar HC-SR04 Sonic Distance Sensor Test and Calibration Program
         Available at http://devestechnet.com/NewTech/SuperSmartCar#ArduinoSoftware
             This program is designed for the HC-SR04 Sonic Distance Sensors
               facing 45 degrees from forward on the front fascia and also
                       on the front Gimball of the SuperSmartCar.

                  Distance 2 to 400cm or 1" to 13 foot Detection Range
                  Digital Output Signal (Nice resolution of .3cm)
                  Detection Angle: 15 Degrees
                  Adjustable detection distance via software(below):
                  
   This is a very accurate sensor that can be used for detecting dropoffs like stairs or the 
    end of a table, etc. It uses a 10uS Pulse Trigger and uses about 15mA each. It works on
   the Sonar Ping principle where the Trigger Pin outputs a pulse, the pulse is bounced off
  an obstacle then the Echo Pin receives that pulse. The time it takes for one pulse to make
   the round trip is then measured and calculated mathematically to determine distance. 

 We decided to use the NewPing Library for more accuracy and better signal response. Be sure 
   to check the Arduino Playground for info:   http://playground.arduino.cc/Code/NewPing 
                       Thank You Tom Eckel for this very special effort!
                       
                       Open Serial Monitor to view Sensor Output */
#include <Servo.h>

Servo ServoPan;  int servoPin1 = 18;              /* to hard code these: ex. int servoPin1 = 8 */
Servo ServoTilt; int servoPin2 = 19;              /* to hard code these: ex. int servoPin1 = 9 */

const int servoDelay = 25;
const float analogMap = 1023;

char topMenu = 'Q';
char subMenuA;
char subMenuB;
char subMenuC;
char subMenuD;
char subMenuE;

bool runOnce = true;
bool setGimbal = true;

#include <NewPing.h>

#define echoPinL 14                                         /*Echo Pin on the Left Front Facia */
#define trigPinL 15                                      /*Trigger Pin on the Left Front Facia */

#define echoPinG 24                                     /* Echo Pin on the Front Servo Gimball */
#define trigPinG 25                                  /* Trigger Pin on the Front Servo Gimball */

#define echoPinR 20                                        /*Echo Pin on the Right Front Facia */
#define trigPinR 21                                     /*Trigger Pin on the Right Front Facia */

#define SONAR_NUM 3                                  /* Number of Sensors on the SuperSmartCar */
#define distanceLow 500                                   /* total distance for Fascia Sensors */
#define distanceHigh 500                                  /* total distance for Gimball Sensor */
int pingInterval = 33;                                           /* Milliseconds between Pings */

unsigned long pingTimer[SONAR_NUM];                     /* When each sensor in the Array Pings */
unsigned int in[SONAR_NUM];                                     /* stores Ping distances in cm */
uint8_t currentSensor = 0;                                           /* Which sensor is active */
NewPing sonar[SONAR_NUM] = {
  NewPing(trigPinR, echoPinR, distanceLow),
  NewPing(trigPinG, echoPinG, distanceHigh),
  NewPing(trigPinL, echoPinL, distanceLow),
};

int panLowLimit = 0;                       /* Set this variable for your Pan Servo's Low Limit */
int panHighLimit = 175;                   /* Set this variable for your Pan Servo's High Limit */

int tiltLowLimit = 0;                     /* Set this variable for your Tilt Servo's Low Limit */
int tiltHighLimit = 176;                 /* Set this variable for your Tilt Servo's High Limit */

void setup() {
  Serial.begin(115200);                                     /* Starts the serial communication */
  pingTimer[0] = millis() + 75;                                      /* First ping start in ms */
  for (uint8_t i = 1; i < SONAR_NUM; i++) {
    pingTimer[i] = pingTimer[i - 1] + pingInterval;
  }
  pinMode (servoPin1, OUTPUT);  pinMode (servoPin2, OUTPUT);
  ServoPan.attach(servoPin1); ServoPan.write((panHighLimit - panLowLimit) / 2);
  ServoTilt.attach(servoPin2); ServoTilt.write((tiltHighLimit - tiltLowLimit) / 2);
}

void mainMenu() {
  if (runOnce == true) {
    Serial.println("\n                    -- DTN's Super SonarMaster Top Menu --");
    Serial.println(" A) Left Sensor  B) Gimbal Sensor  C) Right Sensor  D) QuikTest ALL    E) Info\n");
    runOnce = false;
  }
}

void subAMenu() {
  Serial.println("\n                 -- DTN's Super SonarMaster Left Menu A --");
  Serial.println(" 1) Test Left Sensor (Q Exits)        2) Info             3) Exit to Main Menu\n");
}

void subBMenu() {
  Serial.println("\n                -- DTN's Super SonarMaster Gimbal Menu B --");
  Serial.println(" 1) Test Gimbal Sensor  2) Test w/Servo System  3) Info   4) Exit to Main Menu\n");
}

void subCMenu() {
  Serial.println("\n                -- DTN's Super SonarMaster Right Menu C --");
  Serial.println(" 1) Test Right Sensor            2) Info                  3) Exit to Main Menu\n");
}

void subDMenu() {
  Serial.println("\n               -- DTN's Super SonarMaster QuikTest Menu D --");
  Serial.println(" 1) Test All                     2) Info                  3) Exit to Main Menu\n");
}

void subEMenu() {
  Serial.println("\n          -- DTN's Super SonarMaster Information Menu E --");
  Serial.println(" 1) How to Use this Program                               2) Exit to Main Menu\n");
}

void topCaseA() {
  if (subMenuA != '1' && subMenuA != '2' && subMenuA != '3') {
    subAMenu();                /* if menuA options have not already been selected present Menu */
  }                                                     /* prevents dual Menus from popping up */
}

void topCaseB() {
  if (subMenuB != '1' && subMenuB != '2' && subMenuB != '3' && subMenuB != '4') {
    subBMenu();                /* if menuB options have not already been selected present Menu */
  }                                                     /* prevents dual Menus from popping up */
}

void topCaseC() {
  if (subMenuC != '1' && subMenuC != '2' && subMenuC != '3') {
    subCMenu();                /* if menuC options have not already been selected present Menu */
  }                                                     /* prevents dual Menus from popping up */
}

void topCaseD() {
  if (subMenuD != '1' && subMenuD != '2' && subMenuD != '3') {
    subDMenu();                /* if menuD options have not already been selected present Menu */
  }                                                     /* prevents dual Menus from popping up */
}

void topCaseE() {
  if (subMenuE != '1' && subMenuE != '2') {
    subEMenu();                /* if menuE options have not already been selected present Menu */
  }                                                     /* prevents dual Menus from popping up */
}

void subMenuE1() {
  Serial.println(F(" This program was created to fully test the  HC-SR04  Sonic Sensors on our DTN"));
  Serial.println(F(" SuperSmartCar. The car uses 3 of these sensors. One is pointed 45 degrees off"));
  Serial.println(F(" front center, Two is positioned on the Servo Gimbal system primarily used for"));
  Serial.println(F(" detecting dropoffs such as staircases, etc. The third sensor is pointed at 45"));
  Serial.println(F(" degrees off of front center in the opposite direction."));
  Serial.println(F(""));
  Serial.println(F(" Look for updates to the program and the particulars of this fun project on our"));
  Serial.println(F(" website at http://devestechnet.com/NewTech/SuperSmartCar#ArduinoSoftware.  Our"));
  Serial.println(F(" special thanks to Tim Eckel for authoring  NewPing and to the Arduino.cc Forum"));
  Serial.println(F(" volunteers for their patience."));
  Serial.println(F(""));
  Serial.println(F(" The programs functions are pretty much self explanatory.  Left, Gimbal, Right."));
  Serial.println(F(" are ways to test the individual sensors individually.  The QuikTest(D) feature"));
  Serial.println(F(" tests all three sensors at once displaying the telemetry on the Serial Monitor"));
  Serial.println(F(" The Gimball Sensor  (B)  Menu allows you to sweep the Pan Servo from 45 to 135"));
  Serial.println(F(" degrees while sending the proper telemetry from the sonic sensor to the Serial"));
  Serial.println(F(" Monitor.  This lets you see a dropoff and program your SuperSmartCar to handle"));
  Serial.println(F(" the issue. (Q to Exit)"));
  Serial.println(F(""));
  if (Serial.peek() == 'Q' || Serial.peek() == 'q') {
    while (Serial.available() == 0) {}
    Serial.println("\n Exiting . . .\n");
    topMenu = 'e';
    topCaseE();
  }
}

void echoCheck() {
  if (sonar[currentSensor].check_timer()) {
    in[currentSensor] = sonar[currentSensor].ping_result / US_ROUNDTRIP_IN;
  }
}

void oneSensorCycle() {
  for (int i = 0; i < SONAR_NUM; i++) {
    if (i == 0) {
      Serial.print(" Right Ping ");
    }
    else if (i == 1) {
      Serial.print("\tGimbal Ping ");         /* tab to prevent horizontal shifting of results */
    }
    else if (i = 2) {
      Serial.print ("\tLeft Ping ");
    }
    Serial.print(": ");
    Serial.print(in[i]);
    Serial.print(" Inches     ");
  }
  Serial.println();
}

void subMenuA1() {
  while (subMenuA == '1') {
    if (Serial.peek() == 'q' || Serial.peek() == 'Q') {
      Serial.println("\n Exiting . . .");
      topMenu = 'a';
      topCaseA();
      break;
    }
    NewPing sonar(trigPinL, echoPinL, distanceLow);
    delay(100);
    Serial.print("Left Sensor Sonar Ping: ");
    Serial.print(sonar.ping_in());
    Serial.println(" inches");
  }
}

void subMenuA2() {
  Serial.println(" Reserved for future Telemetry Collection (Q Exits)\n");
  while (subMenuA == '2') {
    if (Serial.peek() == 'q' || Serial.peek() == 'Q') {
      Serial.println("\n Exiting . . .");
      topMenu = 'a';
      topCaseA();
      break;
    }
  }
}

void subMenuB1() {
  while (subMenuB == '1') {
    if (Serial.peek() == 'q' || Serial.peek() == 'Q') {
      Serial.println("\n Exiting . . .");
      topMenu = 'b';
      topCaseB();
      break;
    }
    NewPing sonar(trigPinG, echoPinG, distanceHigh);
    delay(100);
    Serial.print("Gimbal Sensor Sonar Ping: ");
    Serial.print(sonar.ping_in());
    Serial.println(" inches");
  }
}

void subMenuB2() {
  while (subMenuB == '2') {
    int setPosAngle = 30;
    ServoTilt.write(setPosAngle);
    panLowLimit  =  45;
    panHighLimit = 135;
    for (int posPan = panLowLimit; posPan <= panHighLimit; posPan = posPan + 2) {
      ServoPan.write(posPan);
      NewPing sonar(trigPinG, echoPinG, distanceHigh);
      delay(30);
      Serial.print("Gimbal Sensor Sonar Ping: ");

      if (sonar.ping_in() >= 10) {
        Serial.println("DROP OFF IMMINENT!!");
      }
      else {
        Serial.print(sonar.ping_in());
        Serial.println(" inches");
      }
    }
    for (int posPan = panHighLimit; posPan >= panLowLimit; posPan = posPan - 2) {
      ServoPan.write(posPan);
      NewPing sonar(trigPinG, echoPinG, distanceHigh);
      delay(30);
      Serial.print("Gimbal Sensor Sonar Ping: ");

      if (sonar.ping_in() >= 10) {
        Serial.println("DROP OFF IMMINENT!!");
        //allStop() // or whatever you want to do here
      }
      else {
        Serial.print(sonar.ping_in());
        Serial.println(" inches");
      }
    }
    if (Serial.peek() == 'q' || Serial.peek() == 'Q') {
      Serial.println("\n Exiting . . .");
      topMenu = 'b';
      topCaseB();
      break;
    }
  }
}

void subMenuB3() {
  Serial.println(" Reserved for future Telemetry Collection (Q Exits)\n");
  while (subMenuB == '3') {
    if (Serial.peek() == 'q' || Serial.peek() == 'Q') {
      Serial.println("\n Exiting . . .");
      topMenu = 'b';
      topCaseB();
      break;
    }
  }
}

void subMenuC1() {
  while (subMenuC == '1') {
    if (Serial.peek() == 'q' || Serial.peek() == 'Q') {
      Serial.println("\n Exiting . . .");
      topMenu = 'c';
      topCaseC();
      break;
    }
    NewPing sonar(trigPinR, echoPinR, distanceLow);
    delay(100);
    Serial.print("Right Sensor Sonar Ping: ");
    Serial.print(sonar.ping_in());
    Serial.println(" inches");
  }
}

void subMenuC2() {
  Serial.println(" Reserved for future Telemetry Collection (Q Exits)\n");
  while (subMenuC == '2') {
    if (Serial.peek() == 'q' || Serial.peek() == 'Q') {
      Serial.println("\n Exiting . . .");
      topMenu = 'c';
      topCaseC();
      break;
    }
  }
}

void subMenuD1() {
  while (subMenuD == '1') {
    if (Serial.peek() == 'q' || Serial.peek() == 'Q') {
      Serial.println("\n Exiting . . .");
      topMenu = 'd';
      topCaseD();
      break;
    }
    for (uint8_t i = 0; i < SONAR_NUM; i++) {
      if (millis() >= pingTimer[i]) {
        pingTimer[i] += pingInterval * SONAR_NUM;
        if (i == 0 && currentSensor == SONAR_NUM - 1) {
          oneSensorCycle();
        }
        sonar[currentSensor].timer_stop();
        currentSensor = i;
        in[currentSensor] = 0;
        sonar[currentSensor].ping_timer(echoCheck);
      }
    }
  }
}

void subMenuD2() {
  Serial.println(" Reserved for future Telemetry Collection (Q Exits)\n");
  while (subMenuD == '2') {
    if (Serial.peek() == 'q' || Serial.peek() == 'Q') {
      Serial.println("\n Exiting . . .");
      topMenu = 'd';
      topCaseD();
      break;
    }
  }
}

void subMenuExit() {
  runOnce = true;
  topMenu = 'Q';
  Serial.print(" Exiting to Main Menu . . .\n");
  mainMenu();
  return;
}

void loop() {
  if (runOnce == true) {
    mainMenu();
    runOnce = false;
  }
  if (Serial.available() > 0 ) {
    if (topMenu == 'Q') {
      topMenu = Serial.read();
    }
    if (topMenu == 'a' || topMenu == 'A') {
      subMenuA = Serial.read();
    }
    else if (topMenu == 'b' || topMenu == 'B') {
      subMenuB = Serial.read();
    }
    else if (topMenu == 'c' || topMenu == 'C') {
      subMenuC = Serial.read();
    }
    else if (topMenu == 'd' || topMenu == 'D') {
      subMenuD = Serial.read();
    }
    else if (topMenu == 'e' || topMenu == 'E') {
      subMenuE = Serial.read();
    }
    switch (topMenu) {
      case 'A' : case 'a' : {
          topCaseA();
          switch (subMenuA) {
            case '1' : {
                subMenuA1();
                break;
              }
            case '2' : {
                subMenuA2();
                break;
              }
            case '3' : {
                subMenuExit();
                break;
              }
              break;
          } /* End subMenu A Switch */
          break;
        } /* End Case A */
      case 'B' : case 'b' : {
          topCaseB();
          switch (subMenuB) {
            case '1' : {
                subMenuB1();
                break;
              }
            case '2' : {
                subMenuB2();
                break;
              }
            case '3' : {
                subMenuB3();
                break;
              }
            case '4' : {
                subMenuExit();
                break;
              }
              break;
          } /* End subMenu B Switch */
          break;
        } /* End Case B */
      case 'C' : case 'c' : {
          topCaseC();
          switch (subMenuC) {
            case '1' : {
                subMenuC1();
                break;
              }
            case '2' : {
                subMenuC2();
                break;
              }
            case '3' : {
                subMenuExit();
                break;
              }
              break;
          } /* End subMenu C Switch */
          break;
        } /* End Case C */
      case 'D' : case 'd' : {
          topCaseD();
          switch (subMenuD) {
            case '1' : {
                subMenuD1();
                break;
              }
            case '2' : {
                subMenuD2();
                break;
              }
            case '3' : {
                subMenuExit();
                break;
              }
              break;
          } /* End subMenu D Switch */
          break;
        } /* End Case D */
      case 'E' : case 'e' : {
          topCaseE();
          switch (subMenuE) {
            case '1' : {
                subMenuE1();
                break;
              }
            case '2' : {
                subMenuExit();
                break;
              }
              break;
          }
          break;
        } /* End Case E */
      default :
        if (topMenu != 'a' || topMenu != 'b' || topMenu != 'c' || topMenu != 'd' || topMenu != 'e') {
          runOnce = true;
          if (runOnce == true) {
            Serial.println(" Please Choose Options A - E");
            runOnce = false;
            topMenu = 'Q';
          }
        }
    } /* End topMenu Switch */
  } /* End Serial.available */
} /* End Void Loop */







